pub enum EyrosErrorKind {
MetaBitfieldInsufficientBytes {},
ScalarInBounds {},
IntervalSides {
dimension: usize,
min: String,
max: String,
},
TreeRemoved {
id: TreeId,
},
TreeEmpty {
id: TreeId,
file: String,
},
RemoveIdsMissing {
ids: Vec<String>,
},
}
Show fields
Show fields
Show fields
Show fields
Show fields
Show fields
Fields of RemoveIdsMissing
Formats the value using the given formatter. Read more
impl<T> Any for T where
T: 'static + ?Sized, [src]
Immutably borrows from an owned value. Read more
Mutably borrows from an owned value. Read more
impl<T, U> Into<U> for T where
U: From<T>, [src]
impl<T, U> TryFrom<U> for T where
U: Into<T>, [src]
The type returned in the event of a conversion error.
The type returned in the event of a conversion error.